==> tecfy_corev2_package-1.1.51.tar.gz: lib/views/components/entity/many_to_many_with_data.dart <==
part of tecfy_corev2_client_package;

class ManyToManyWithData extends StatefulWidget {
  final Entity? entity;
  final String? entityId;
  final String entityMiddle;
  // final String propertieMiddle;
  final List<dynamic>? items;
  final String? label;
  final bool isRequired;
  final void Function(List<dynamic>)? onChanged;
  ManyToManyWithData(this.entityId,
      {this.entity,
      this.isRequired = false,
      this.items,
      this.onChanged,
      this.label,
      required this.entityMiddle});
  @override
  _ManyToManyWithDataState createState() => _ManyToManyWithDataState();
}

class _ManyToManyWithDataState extends TecfyState<ManyToManyWithData> {
  Entity? _entity;
  // List<Property> properties;
  List<Map<String, dynamic>>? _itemList;
  List<dynamic>? _items;
  @override
  void initState() {
    if (kDebugMode)
      print(
          '--------------_ManyToManyWithDataState--------------- ${widget.entityId}');
    if (widget.entity == null) {
      if (kDebugMode) {
        EntityService.entities!.keys.forEach((element) {
          print(element);
        });
      }
    }
    _entity = EntityService.getEntity(widget.entityMiddle);
    if (_entity == null) {
      if (kDebugMode) print('entity not found !!!      ${widget.entityMiddle}');
      return;
    }
    _loadData();
    super.initState();
  }

  _onChanged(Map<String, dynamic> data) {
    // save to selected items
    if (_items == null) {
      print('XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX items is null');
      print(_items);
      print(widget.items);
      return;
    }
    var index = _items?.indexWhere((item) => item['_id'] == data['_id']) ?? -1;
    if (index >= 0) {
      _items?.replaceRange(index, index + 1, [data]);
    } else {
      _items?.add(data);
    }
    widget.onChanged?.call(_items ?? []);
  }

  _loadData() async {
    if (_items != null && _items!.length == 0 && widget.items == null) return;
    _items = widget.items ?? [];
    var result = await EntityService.getData(widget.entity, allRecords: true);
    if (result.data == null || result.data!.length == 0)
      result = await EntityService.getData(widget.entity, allRecords: true);
    if (!result.success) {
      result.showMessage(context);
      return;
    }
    _itemList = [];
    for (Map<String, dynamic> row in result.data!) {
      var otherRows =
          widget.items?.where((s) => s['_id'] == row['_id']).toList() ?? [];

      // console.log('this.isFilterData',this.isFilterData);
      if (otherRows.length > 0) {
        // itemList.add({'selected' :true, ...otherRows[0],id:row.id,display:row.display});
        var r = otherRows[0];
        r['selected'] = true;
        r['_id'] = row['_id'];
        //r[widget.entity?.dbName ?? 'relation'] = row;

        this._itemList?.add(r);
      } else {
        // row[widget.propertieMiddle] = row['_id'];
        row['_id'] = row['_id'];

        //row[widget.entity?.dbName ?? 'relation'] = row;

        row['selected'] = false;
        this._itemList?.add(row);
      }
    }
    if (mounted) setState(() {});
  }

  @override
  Widget build(BuildContext context) {
    if (_items != widget.items) _loadData();
    return _itemList == null
        ? Center(child: AppNavigator.getLoadingWidget(70))
        : LayoutService.isMobile(context)
            ? _getMobileWidget()
            : _getDesktopWidget();
  }

  Widget _getDesktopWidget() {
    return SingleChildScrollView(
        scrollDirection: Axis.horizontal,
        child: DataTable(
            showCheckboxColumn: true,
            checkboxHorizontalMargin: 10,
            dividerThickness: 0.1,
            onSelectAll: (v) {
              for (var d in _itemList!) {
                d['selected'] = v ?? false;
                if (!d['selected']) {
                  _items?.removeWhere((item) => item['_id'] == d['_id']);
                  widget.onChanged?.call(_items ?? []);
                }
              }
              setState(() {});
            },
            columns: _entity!.editProperties
                .map<DataColumn>((e) =>
                    DataColumn(label: TecfyText(e.nameAr), tooltip: e.dbName))
                .toList()
              ..insert(0, DataColumn(label: TecfyText(trans('Description')))),
            rows: _itemList!
                .map<DataRow>(
                  (d) => DataRow(
                    selected: d['selected'] ?? false,
                    onSelectChanged: (v) {
                      d['selected'] = v ?? false;
                      if (!d['selected']) {
                        _items?.removeWhere((item) => item['_id'] == d['_id']);
                        widget.onChanged?.call(_items ?? []);
                      }
                      setState(() {});
                    },
                    cells: _entity!.editProperties
                        .map<DataCell>((e) => DataCell(!d['selected']
                            ? Container()
                            : PorpertyComponent(
                                property: e,
                                data: d,
                                noLabel: true,
                                entity: _entity,
                                onChange: (v) {
                                  d[e.dbName] = v;
                                  _onChanged(d);
                                },
                                readOnly: !d['selected'],
                              )))
                        .toList()
                      ..insert(
                          0,
                          DataCell(TecfyText(d['display'] ??
                              d['name']?['ar'] ??
                              d['title']?['ar'] ??
                              'N/A'))),
                  ),
                )
                .toList()));
  }

  Widget _getMobileWidget() {
    return Column(
      children: _itemList
              ?.map((d) => Column(
                    children: <Widget>[
                      AppCheckBox(
                          initalValue: d['selected'] ?? false,
                          label: d['display'] ?? '!!',
                          onChanged: (v) {
                            d['selected'] = v;
                            if (!v) {
                              _items?.removeWhere(
                                  (item) => item['_id'] == d['_id']);
                              widget.onChanged?.call(_items ?? []);
                            }
                            setState(() {});
                          }),
                      d['selected'] == true
                          ? InLineForm(
                              widget.entityId,
                              false,
                              data: d,
                              entity: _entity,
                              onChanged: (v) => _onChanged(d),
                            )
                          : Container()
                    ],
                  ))
              .toList() ??
          [],
    );
  }
}

==> tecfy_corev2_package-1.1.57.tar.gz: lib/views/components/entity/many_to_many_with_data.dart <==
part of tecfy_corev2_client_package;

class ManyToManyWithData extends StatefulWidget {
  final Entity? entity;
  final String? entityId;
  final String entityMiddle;
  // final String propertieMiddle;
  final List<dynamic>? items;
  final String? label;
  final bool isRequired;
  final void Function(List<dynamic>)? onChanged;
  ManyToManyWithData(this.entityId,
      {this.entity,
      this.isRequired = false,
      this.items,
      this.onChanged,
      this.label,
      required this.entityMiddle});
  @override
  _ManyToManyWithDataState createState() => _ManyToManyWithDataState();
}

class _ManyToManyWithDataState extends TecfyState<ManyToManyWithData> {
  Entity? _entity;
  // List<Property> properties;
  List<Map<String, dynamic>>? _itemList;
  List<dynamic>? _items;
  @override
  void initState() {
    if (kDebugMode)
      print(
          '--------------_ManyToManyWithDataState--------------- ${widget.entityId}');
    if (widget.entity == null) {
      if (kDebugMode) {
        EntityService.entities!.keys.forEach((element) {
          print(element);
        });
      }
    }
    _entity = EntityService.getEntity(widget.entityMiddle);
    if (_entity == null) {
      if (kDebugMode) print('entity not found !!!      ${widget.entityMiddle}');
      return;
    }
    _loadData();
    super.initState();
  }

  _onChanged(Map<String, dynamic> data) {
    // save to selected items
    if (_items == null) {
      print('XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX items is null');
      print(_items);
      print(widget.items);
      return;
    }
    var index = _items?.indexWhere((item) => item['_id'] == data['_id']) ?? -1;
    if (index >= 0) {
      _items?.replaceRange(index, index + 1, [data]);
    } else {
      _items?.add(data);
    }
    widget.onChanged?.call(_items ?? []);
  }

  _loadData() async {
    if (_items != null && _items!.length == 0 && widget.items == null) return;
    _items = widget.items ?? [];
    var result = await EntityService.getData(widget.entity, allRecords: true);
    if (result.data == null || result.data!.length == 0)
      result = await EntityService.getData(widget.entity, allRecords: true);
    if (!result.success) {
      result.showMessage(context);
      return;
    }
    _itemList = [];
    for (Map<String, dynamic> row in result.data!) {
      var otherRows =
          widget.items?.where((s) => s['_id'] == row['_id']).toList() ?? [];

      // console.log('this.isFilterData',this.isFilterData);
      if (otherRows.length > 0) {
        // itemList.add({'selected' :true, ...otherRows[0],id:row.id,display:row.display});
        var r = otherRows[0];
        r['selected'] = true;
        r['_id'] = row['_id'];
        //r[widget.entity?.dbName ?? 'relation'] = row;

        this._itemList?.add(r);
      } else {
        // row[widget.propertieMiddle] = row['_id'];
        row['_id'] = row['_id'];

        //row[widget.entity?.dbName ?? 'relation'] = row;

        row['selected'] = false;
        this._itemList?.add(row);
      }
    }
    if (mounted) setState(() {});
  }

  @override
  Widget build(BuildContext context) {
    if (_items != widget.items) _loadData();
    return _itemList == null
        ? Center(child: AppNavigator.getLoadingWidget(70))
        : LayoutService.isMobile(context) ||
                widget.entity!.properties
                    .any((element) => element is PropertyOtherEntityMany)
            ? _getMobileWidget()
            : _getDesktopWidget();
  }

  Widget _getDesktopWidget() {
    return SingleChildScrollView(
        scrollDirection: Axis.horizontal,
        child: DataTable(
            showCheckboxColumn: true,
            checkboxHorizontalMargin: 10,
            dividerThickness: 0.1,
            onSelectAll: (v) {
              for (var d in _itemList!) {
                d['selected'] = v ?? false;
                if (!d['selected']) {
                  _items?.removeWhere((item) => item['_id'] == d['_id']);
                  widget.onChanged?.call(_items ?? []);
                }
              }
              setState(() {});
            },
            columns: _entity!.editProperties
                .map<DataColumn>((e) =>
                    DataColumn(label: TecfyText(e.nameAr), tooltip: e.dbName))
                .toList()
              ..insert(0, DataColumn(label: TecfyText(trans('Description')))),
            rows: _itemList!
                .map<DataRow>(
                  (d) => DataRow(
                    selected: d['selected'] ?? false,
                    onSelectChanged: (v) {
                      d['selected'] = v ?? false;
                      if (!d['selected']) {
                        _items?.removeWhere((item) => item['_id'] == d['_id']);
                        widget.onChanged?.call(_items ?? []);
                      }
                      setState(() {});
                    },
                    cells: _entity!.editProperties
                        .map<DataCell>((e) => DataCell(!d['selected']
                            ? Container()
                            : PorpertyComponent(
                                property: e,
                                data: d,
                                noLabel: true,
                                entity: _entity,
                                onChange: (v) {
                                  d[e.dbName] = v;
                                  _onChanged(d);
                                },
                                readOnly: !d['selected'],
                              )))
                        .toList()
                      ..insert(
                          0,
                          DataCell(TecfyText(d['display'] ??
                              d['name']?['ar'] ??
                              d['title']?['ar'] ??
                              'N/A'))),
                  ),
                )
                .toList()));
  }

  Widget _getMobileWidget() {
    print(_itemList);
    return Column(
      children: _itemList
              ?.map((d) => Column(
                    children: <Widget>[
                      AppCheckBox(
                          initalValue: d['selected'] ?? false,
                          label: MapService.getValue(d, "name")[
                                  LocalConfigService.languagePrefix
                                      .toLowerCase()] ??
                              ''
                                  '!!',
                          onChanged: (v) {
                            d['selected'] = v;
                            if (!v) {
                              _items?.removeWhere(
                                  (item) => item['_id'] == d['_id']);
                              widget.onChanged?.call(_items ?? []);
                            }
                            setState(() {});
                          }),
              d['selected'] == true
                  ? InLineForm(
                widget.entityId,
                false,
                data: d,
                entity: _entity,
                onChanged: (v) => _onChanged(d),
              )
                  : Container()
            ],
          ))
          .toList() ??
          [],
    );
  }
}
